/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright (c) 2017 Oracle and/or its affiliates. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License.  You can
 * obtain a copy of the License at
 * https://oss.oracle.com/licenses/CDDL+GPL-1.1
 * or LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 *
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at LICENSE.txt.
 *
 * GPL Classpath Exception:
 * Oracle designates this particular file as subject to the "Classpath"
 * exception as provided by Oracle in the GPL Version 2 section of the License
 * file that accompanied this code.
 *
 * Modifications:
 * If applicable, add the following below the License Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyright [year] [name of copyright owner]"
 *
 * Contributor(s):
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */

package com.sun.s1asdev.ejb.ejb30.persistence.extendedem;

import javax.ejb.Stateful;
import javax.persistence.PersistenceContextType;
import javax.persistence.PersistenceContext;
import javax.persistence.EntityNotFoundException;

import javax.persistence.EntityManager;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;

@Stateful
public class SfulBean
    implements Sful {

    private String name;

    private @PersistenceContext(
            unitName="lib/ejb-ejb30-persistence-extendedem-par1.jar#em",
            type=PersistenceContextType.EXTENDED) EntityManager extendedEM;

    private @PersistenceContext(
            unitName="lib/ejb-ejb30-persistence-extendedem-par3.jar#em")
            EntityManager txEM;

    private transient Person _p;

    public void createPerson(String name) {

        _p = new Person(name);
        extendedEM.persist(_p);
        System.out.println("Created " + _p);
            this.name = name;
    }

    public Person findPerson() {

        Person p = extendedEM.find(Person.class, name);
        System.out.println("Found " + p);
        return p;
    }

    public boolean removePerson() {

        Person p = txEM.find(Person.class, name);
        boolean removed = false;
        if (p != null) {
            txEM.remove(p);
            removed = true;
        }
        return removed;
    }

    @TransactionAttribute(TransactionAttributeType.NOT_SUPPORTED)
    public Person nonTxFindPerson() {
        Person p = (Person) txEM.find(Person.class, name);
        System.out.println("Found " + _p);
        return p;
    }


    public boolean refreshAndFindPerson() {
        boolean foundAfterRefresh = true;

        Person pp = extendedEM.find(Person.class, name);
        foundAfterRefresh = (pp == null);
        System.out.println("Error: Found " + pp);
        return foundAfterRefresh;
    }

}
